@@ -16,7 +16,6 @@
 #' the markdown syntax for citations in the
 #' \href{https://rmarkdown.rstudio.com/authoring_bibliographies_and_citations.html}{Bibliographies
 #' and Citations} article in the online documentation.
-#' @inheritParams output_format
 #' @inheritParams pdf_document
 #' @inheritParams html_document
 #' @param toc \code{TRUE} to include a table of contents in the output (only
@@ -64,6 +63,7 @@ beamer_presentation <- function(toc = FALSE,
                                 highlight = "default",
                                 template = "default",
                                 keep_tex = FALSE,
+                                keep_md = FALSE,
                                 latex_engine = "pdflatex",
                                 citation_package = c("none", "natbib", "biblatex"),
                                 self_contained = TRUE,
@@ -154,6 +154,7 @@ beamer_presentation <- function(toc = FALSE,
     pre_processor = pre_processor,
     intermediates_generator = intermediates_generator,
     clean_supporting = !keep_tex,
+    keep_md = keep_md,
     df_print = df_print
   )
 }
