@@ -2,6 +2,12 @@
 #'
 #' Convert documents to and from various formats using the pandoc utility.
 #'
+#' Supported input and output formats are described in the
+#' \href{http://johnmacfarlane.net/pandoc/README.html}{pandoc user guide}.
+#'
+#' The system path as well as the version of pandoc shipped with RStudio (if
+#' running under RStudio) are scanned for pandoc and the highest version
+#' available is used.
 #' @param input Character vector containing paths to input files
 #'   (files must be UTF-8 encoded)
 #' @param to Format to convert to (if not specified, you must specify
@@ -9,21 +15,13 @@
 #' @param from Format to convert from (if not specified then the format is
 #'   determined based on the file extension of \code{input}).
 #' @param output Output file (if not specified then determined based on format
-#'   being converted to)
+#'   being converted to).
 #' @param citeproc \code{TRUE} to run the pandoc-citeproc filter (for processing
-#'   citations) as part of the conversion
+#'   citations) as part of the conversion.
 #' @param options Character vector of command line options to pass to pandoc.
 #' @param verbose \code{TRUE} to show the pandoc command line which was executed
 #' @param wd Working directory in which code will be executed. If not
-#'   supplied, defaults to the common base directory of \code{input}
-#'
-#' @details Supported input and output formats are described in the
-#'   \href{http://johnmacfarlane.net/pandoc/README.html}{pandoc user guide}.
-#'
-#'   The system path as well as the version of pandoc shipped with RStudio (if
-#'   running under RStudio) are scanned for pandoc and the highest version
-#'   available is used.
-#'
+#'   supplied, defaults to the common base directory of \code{input}.
 #' @examples
 #' \dontrun{
 #' library(rmarkdown)
@@ -38,7 +36,6 @@
 #' # add some pandoc options
 #' pandoc_convert("input.md", to="pdf", options = c("--listings"))
 #' }
-#'
 #' @export
 pandoc_convert <- function(input,
                            to = NULL,
@@ -56,10 +53,10 @@ pandoc_convert <- function(input,
   if (is.null(wd)) {
     wd <- base_dir(input)
   }
+
   oldwd <- setwd(wd)
   on.exit(setwd(oldwd), add = TRUE)
 
-
   # input file and formats
   args <- c(input)
   if (!is.null(to)) {
@@ -69,7 +66,7 @@ pandoc_convert <- function(input,
   if (!is.null(from))
     args <- c(args, "--from", from)
 
-  #  output file
+  # output file
   if (!is.null(output))
     args <- c(args, "--output", output)
 
@@ -105,29 +102,72 @@ pandoc_convert <- function(input,
   invisible(NULL)
 }
 
+
+#' Convert a bibliograpy file
+#'
+#' Convert a bibliography file (e.g. a BibTeX file) to an R list, JSON text,
+#'   or YAML text
+#'
+#' @param file Bibliography file
+#' @param type Conversion type
+#'
+#' @return For `type = "list"`, and R list. For `type = "json"` or `type = "yaml"`,
+#'   a character vector with the specified format.
+#'
+#' @export
+pandoc_citeproc_convert <- function(file, type = c("list", "json", "yaml")) {
+
+  # ensure we've scanned for pandoc
+  find_pandoc()
+
+  # resolve type
+  type <- match.arg(type)
+
+  # build the conversion command
+  conversion <- switch(type,
+    list = "--bib2json",
+    json = "--bib2json",
+    yaml = "--bib2yaml"
+  )
+  args <- c(conversion, file)
+  command <- paste(quoted(pandoc_citeproc()), paste(quoted(args), collapse = " "))
+
+  # run the conversion
+  with_pandoc_safe_environment({
+    result <- system(command, intern = TRUE)
+  })
+  status <- attr(result, "status")
+  if (!is.null(status)) {
+    cat(result, sep = "\n")
+    stop("Error ", status, " occurred building shared library.")
+  }
+
+  # convert the output if requested
+  if (type == "list") {
+    jsonlite::fromJSON(result, simplifyVector = FALSE)
+  } else {
+    result
+  }
+}
+
 #' Check pandoc availability and version
 #'
 #' Determine whether pandoc is currently available on the system (optionally
 #' checking for a specific version or greater). Determine the specific version
 #' of pandoc available.
 #'
-#' @param version Required version of pandoc
-#' @param error Whether to signal an error if pandoc with the required version
-#'   is not found
-#'
-#' @return \code{pandoc_available} returns a logical indicating whether the
-#'   required version of pandoc is available. \code{pandoc_version} returns a
-#'   \code{\link[base]{numeric_version}} with the version of pandoc found.
-#'
-#' @details
-#'
 #' The system environment variable \samp{PATH} as well as the version of pandoc
 #' shipped with RStudio (its location is set via the environment variable
 #' \samp{RSTUDIO_PANDOC} by RStudio products like the RStudio IDE, RStudio
 #' Server, Shiny Server, and RStudio Connect, etc) are scanned for pandoc and
 #' the highest version available is used. Please do not modify the environment
 #' variable \samp{RSTUDIO_PANDOC} unless you know what it means.
-#'
+#' @param version Required version of pandoc
+#' @param error Whether to signal an error if pandoc with the required version
+#'   is not found
+#' @return \code{pandoc_available} returns a logical indicating whether the
+#'   required version of pandoc is available. \code{pandoc_version} returns a
+#'   \code{\link[base]{numeric_version}} with the version of pandoc found.
 #' @examples
 #' \dontrun{
 #' library(rmarkdown)
@@ -139,7 +179,8 @@ pandoc_convert <- function(input,
 #'   cat("required version of pandoc is available!\n")
 #' }
 #' @export
-pandoc_available <- function(version = NULL, error = FALSE) {
+pandoc_available <- function(version = NULL,
+                             error = FALSE) {
 
   # ensure we've scanned for pandoc
   find_pandoc()
@@ -168,10 +209,12 @@ pandoc_version <- function() {
 #'
 #' Functions that assist in creating various types of pandoc command line
 #' arguments (e.g. for templates, table of contents, highlighting, and content
-#' includes)
+#' includes).
 #'
+#' Non-absolute paths for resources referenced from the
+#' \code{in_header}, \code{before_body}, and \code{after_body}
+#' parameters are resolved relative to the directory of the input document.
 #' @inheritParams includes
-#'
 #' @param name Name of template variable to set.
 #' @param value Value of template variable (defaults to \code{true} if missing).
 #' @param toc \code{TRUE} to include a table of contents in the output.
@@ -181,34 +224,28 @@ pandoc_version <- function() {
 #'   "pdflatex", "lualatex", and "xelatex".
 #' @param default The highlighting theme to use if "default"
 #'   is specified.
-#'
-#' @return A character vector with pandoc command line arguments
-#'
-#' @details Non-absolute paths for resources referenced from the
-#'   \code{in_header}, \code{before_body}, and \code{after_body}
-#'   parameters are resolved relative to the directory of the input document.
-#'
+#' @return A character vector with pandoc command line arguments.
 #' @examples
 #' \dontrun{
-#'
 #' library(rmarkdown)
 #'
 #' pandoc_include_args(before_body = "header.htm")
 #' pandoc_include_args(before_body = "header.tex")
 #'
-#' pancoc_highlight_args("kate")
+#' pandoc_highlight_args("kate")
 #'
 #' pandoc_latex_engine_args("pdflatex")
 #'
 #' pandoc_toc_args(toc = TRUE, toc_depth = 2)
-#'
 #' }
 #' @name pandoc_args
 NULL
 
 #' @rdname pandoc_args
 #' @export
-pandoc_variable_arg <- function(name, value) {
+pandoc_variable_arg <- function(name,
+                                value) {
+
   c("--variable", if (missing(value)) name else paste(name, "=", value, sep = ""))
 }
 
@@ -234,7 +271,8 @@ pandoc_include_args <- function(in_header = NULL,
 
 #' @rdname pandoc_args
 #' @export
-pandoc_highlight_args <- function(highlight, default = "tango") {
+pandoc_highlight_args <- function(highlight,
+                                  default = "tango") {
 
   args <- c()
 
@@ -252,6 +290,7 @@ pandoc_highlight_args <- function(highlight, default = "tango") {
 #' @rdname pandoc_args
 #' @export
 pandoc_latex_engine_args <- function(latex_engine) {
+
   c(if (pandoc2.0()) "--pdf-engine" else "--latex-engine",
     find_latex_engine(latex_engine))
 }
@@ -260,6 +299,7 @@ pandoc_latex_engine_args <- function(latex_engine) {
 # of the PATH environment variable by OSX 10.10 Yosemite prevents
 # pandoc from finding the engine in e.g. /usr/texbin
 find_latex_engine <- function(latex_engine) {
+
   # do not need full path if latex_engine is available from PATH
   if (!is_osx() || nzchar(Sys.which(latex_engine))) return(latex_engine)
   # resolve path if it's not already an absolute path
@@ -270,7 +310,8 @@ find_latex_engine <- function(latex_engine) {
 
 #' @rdname pandoc_args
 #' @export
-pandoc_toc_args <- function(toc, toc_depth = 3) {
+pandoc_toc_args <- function(toc,
+                            toc_depth = 3) {
 
   args <- c()
 
@@ -289,14 +330,11 @@ pandoc_toc_args <- function(toc, toc_depth = 3) {
 #' \code{\link[base:path.expand]{path.expand}} on all platforms. On Windows,
 #' transform it to a short path name if it contains spaces, and then convert
 #' forward slashes to back slashes (as required by pandoc for some path
-#' references)
-#'
+#' references).
 #' @param path Path to transform
 #' @param backslash Whether to replace forward slashes in \code{path} with
-#'   backslashes on Windows
-#'
-#' @return Transformed path that can be passed to pandoc on the command line
-#'
+#'   backslashes on Windows.
+#' @return Transformed path that can be passed to pandoc on the command line.
 #' @export
 pandoc_path_arg <- function(path, backslash = TRUE) {
 
@@ -320,14 +358,12 @@ pandoc_path_arg <- function(path, backslash = TRUE) {
 #'
 #' Use the pandoc templating engine to render a text file. Substitutions are
 #' done using the \code{metadata} list passed to the function.
-#'
 #' @param metadata A named list containing metadata to pass to template.
 #' @param template Path to a pandoc template.
 #' @param output Path to save output.
 #' @param verbose \code{TRUE} to show the pandoc command line which was
 #'   executed.
 #' @return (Invisibly) The path of the generated file.
-#'
 #' @export
 pandoc_template <- function(metadata, template, output, verbose = FALSE) {
 
@@ -350,12 +386,9 @@ pandoc_template <- function(metadata, template, output, verbose = FALSE) {
 #'
 #' Create a self-contained HTML document by base64 encoding images,
 #' scripts, and stylesheets referred by the input document.
-#'
 #' @param input Input html file to create self-contained version of.
 #' @param output Path to save output.
-#'
 #' @return (Invisibly) The path of the generated file.
-#'
 #' @export
 pandoc_self_contained_html <- function(input, output) {
 
@@ -370,7 +403,7 @@ pandoc_self_contained_html <- function(input, output) {
   # create a simple body-only template
   template <- tempfile(fileext = ".html")
   on.exit(unlink(template), add = TRUE)
-  writeLines("$body$", template)
+  write_utf8("$body$", template)
 
   # convert from markdown to html to get base64 encoding
   # (note there is no markdown in the source document but
@@ -400,8 +433,8 @@ pandoc_self_contained_html <- function(input, output) {
 }
 
 
-
 validate_self_contained <- function(mathjax) {
+
   if (identical(mathjax, "local"))
     stop("Local MathJax isn't compatible with self_contained\n",
          "(you should set self_contained to FALSE)", call. = FALSE)
@@ -435,9 +468,7 @@ pandoc_mathjax_args <- function(mathjax,
       args <- c(args, "--mathjax")
       args <- c(args, "--variable", paste0("mathjax-url:", mathjax))
     } else if (!self_contained) {
-      args <- c(args, "--mathjax")
-      if (!is.null(mathjax))
-        args <- c(args, mathjax)
+      args <- c(args, paste(c("--mathjax", mathjax), collapse = "="))
     } else {
       warning("MathJax doesn't work with self_contained when not ",
               "using the rmarkdown \"default\" template.", call. = FALSE)
@@ -469,6 +500,7 @@ pandoc_mathjax_local_path <- function() {
 
 
 unix_mathjax_path <- function() {
+
   if (identical(.Platform$OS.type, "unix")) {
     mathjax_path <- "/usr/share/javascript/mathjax"
     if (file.exists(file.path(mathjax_path, "MathJax.js")))
@@ -481,7 +513,8 @@ unix_mathjax_path <- function() {
 }
 
 
-pandoc_html_highlight_args <- function(template, highlight) {
+pandoc_html_highlight_args <- function(template,
+                                       highlight) {
 
   args <- c()
 
@@ -512,54 +545,49 @@ is_highlightjs <- function(highlight) {
 }
 
 # Scan for a copy of pandoc and set the internal cache if it's found.
-find_pandoc <- function() {
+find_pandoc <- function(cache = TRUE) {
 
-  if (is.null(.pandoc$dir)) {
+  if (!is.null(.pandoc$dir) && cache) return(invisible(as.list(.pandoc)))
 
-    # define potential sources
-    sys_pandoc <- find_program("pandoc")
-    sources <- c(Sys.getenv("RSTUDIO_PANDOC"),
-                 ifelse(nzchar(sys_pandoc), dirname(sys_pandoc), ""))
-    if (!is_windows())
-      sources <- c(sources, path.expand("~/opt/pandoc"))
+  # define potential sources
+  sys_pandoc <- find_program("pandoc")
+  sources <- c(Sys.getenv("RSTUDIO_PANDOC"), if (nzchar(sys_pandoc)) dirname(sys_pandoc))
+  if (!is_windows()) sources <- c(sources, path.expand("~/opt/pandoc"))
 
-    # determine the versions of the sources
-    versions <- lapply(sources, function(src) {
-      if (dir_exists(src))
-        get_pandoc_version(src)
-      else
-        numeric_version("0")
-    })
+  # determine the versions of the sources
+  versions <- lapply(sources, function(src) {
+    if (dir_exists(src)) get_pandoc_version(src) else numeric_version("0")
+  })
 
-    # find the maximum version
-    found_src <- NULL
-    found_ver <- numeric_version("0")
-    for (i in 1:length(sources)) {
-      ver <- versions[[i]]
-      if (ver > found_ver) {
-        found_ver <- ver
-        found_src <- sources[[i]]
-      }
-    }
-
-    # did we find a version?
-    if (!is.null(found_src)) {
-      .pandoc$dir <- found_src
-      .pandoc$version <- found_ver
+  # find the maximum version
+  found_src <- NULL
+  found_ver <- numeric_version("0")
+  for (i in seq_along(sources)) {
+    ver <- versions[[i]]
+    if (ver > found_ver) {
+      found_ver <- ver
+      found_src <- sources[[i]]
     }
   }
+
+  # did we find a version?
+  if (!is.null(found_src)) {
+    .pandoc$dir <- found_src
+    .pandoc$version <- found_ver
+  }
+
+  invisible(as.list(.pandoc))
 }
 
 # Get an S3 numeric_version for the pandoc utility at the specified path
 get_pandoc_version <- function(pandoc_dir) {
-  pandoc_path <- file.path(pandoc_dir, "pandoc")
-  if (is_windows()) pandoc_path <- paste0(pandoc_path, ".exe")
-  if (!utils::file_test("-x", pandoc_path)) return(numeric_version("0"))
-  with_pandoc_safe_environment({
-    version_info <- system(paste(shQuote(pandoc_path), "--version"),
-                           intern = TRUE)
-  })
-  version <- strsplit(version_info, "\n")[[1]][1]
+  path <- file.path(pandoc_dir, "pandoc")
+  if (is_windows()) path <- paste0(path, ".exe")
+  if (!utils::file_test("-x", path)) return(numeric_version("0"))
+  info <- with_pandoc_safe_environment(
+    system(paste(shQuote(path), "--version"), intern = TRUE)
+  )
+  version <- strsplit(info, "\n")[[1]][1]
   version <- strsplit(version, " ")[[1]][2]
   numeric_version(version)
 }
@@ -568,31 +596,39 @@ get_pandoc_version <- function(pandoc_dir) {
 # see: https://github.com/rstudio/rmarkdown/issues/31
 # see: https://ghc.haskell.org/trac/ghc/ticket/7344
 with_pandoc_safe_environment <- function(code) {
+
   lc_all <- Sys.getenv("LC_ALL", unset = NA)
+
   if (!is.na(lc_all)) {
     Sys.unsetenv("LC_ALL")
     on.exit(Sys.setenv(LC_ALL = lc_all), add = TRUE)
   }
+
   lc_ctype <- Sys.getenv("LC_CTYPE", unset = NA)
+
   if (!is.na(lc_ctype)) {
     Sys.unsetenv("LC_CTYPE")
     on.exit(Sys.setenv(LC_CTYPE = lc_ctype), add = TRUE)
   }
+
   if (Sys.info()['sysname'] == "Linux" &&
         is.na(Sys.getenv("HOME", unset = NA))) {
     stop("The 'HOME' environment variable must be set before running Pandoc.")
   }
+
   if (Sys.info()['sysname'] == "Linux" &&
         is.na(Sys.getenv("LANG", unset = NA))) {
     # fill in a the LANG environment variable if it doesn't exist
     Sys.setenv(LANG = detect_generic_lang())
     on.exit(Sys.unsetenv("LANG"), add = TRUE)
   }
+
   if (Sys.info()['sysname'] == "Linux" &&
     identical(Sys.getenv("LANG"), "en_US")) {
     Sys.setenv(LANG = "en_US.UTF-8")
     on.exit(Sys.setenv(LANG = "en_US"), add = TRUE)
   }
+
   force(code)
 }
 
@@ -618,25 +654,40 @@ detect_generic_lang <- function() {
 }
 
 
-
 # get the path to the pandoc binary
 pandoc <- function() {
   find_pandoc()
   file.path(.pandoc$dir, "pandoc")
 }
 
+
 # get the path to the pandoc-citeproc binary
 pandoc_citeproc <- function() {
   find_pandoc()
-  citeproc_path = file.path(.pandoc$dir, "pandoc-citeproc")
-  if (file.exists(citeproc_path))
-    citeproc_path
-  else
-    "pandoc-citeproc"
+  bin <- "pandoc-citeproc"
+  p <- file.path(.pandoc$dir, bin)
+  if (xfun::is_windows()) p <- xfun::with_ext(p, "exe")
+  if (file.exists(p)) p else bin
 }
 
+pandoc_lua_filters <- function(...) {
+  args <- c()
+  # lua filters was introduced in pandoc 2.0
+  if (pandoc2.0()) {
+    args <- c(
+      rbind(
+        "--lua-filter",
+        rmarkdown_system_file("rmd", "lua", ...)
+      )
+    )
+  }
+  args
+}
+
+
 # quote args if they need it
 quoted <- function(args) {
+
   # some characters are legal in filenames but without quoting are likely to be
   # interpreted by the shell (e.g. redirection, wildcard expansion, etc.) --
   # wrap arguments containing these characters in quotes.
@@ -646,12 +697,15 @@ quoted <- function(args) {
 }
 
 find_pandoc_theme_variable <- function(args) {
+
   range <- length(args) - 1
+
   for (i in 1:range) {
     if (args[[i]] == "--variable" && grepl("^theme:", args[[i + 1]])) {
       return(substring(args[[i + 1]], nchar("theme:") + 1))
     }
   }
+
   # none found, return NULL
   NULL
 }
@@ -662,4 +716,19 @@ find_pandoc_theme_variable <- function(args) {
 .pandoc$dir <- NULL
 .pandoc$version <- NULL
 
-pandoc2.0 <- function() pandoc_available("2.0")
+pandoc2.0 <- function() {
+  pandoc_available("2.0")
+}
+
+#' Get the path of the pandoc executable
+#'
+#' Returns the path of the pandoc executable used by functions in the the
+#' \pkg{rmarkdown} package. This is the most recent version of pandoc found in
+#' either the system path or shipped with RStudio.
+#'
+#' See the
+#' \href{http://pandoc.org/MANUAL.html}{pandoc manual}
+#' for pandoc commands.
+#'
+#' @export
+pandoc_exec <- pandoc
